# FrequencySweep.jl

Welcome to the FrequencySweep.jl's documentation!  The FrequencySweep.jl library is targeted at the estimation of frequency response functions for systems of differential equations where the analytical representation is difficult to obtain.  

The primary concept behind the operation of this library is rather straight forward.  It is assumed that the solution to the user-defined system of differential equations can be obtained numerically for a specified oscillation frequency of a harmonic forcing function.  It is also assumed that the solution at each frequency point will eventually tend towards a steady-state response where any transient behavior has decayed to such a point as to be irrelevant.  Once this steady-state condition is obtained the FrequencySweep library simply determines the amplitude and phase of the solution.  The final conditions of the solution at a particular frequency point are used as the initial conditions for the next frequency point thereby providing a frequency sweeping behavior.  This sweeping behavior is oftentimes necessary to observe interesting nonlinear phenomenon such as jump phenomenon.  A classical system illustrating jump phenomenon is the [Duffing equation](https://en.wikipedia.org/wiki/Duffing_equation).  The frf_sweep function is used to numerically estimate the frequency response functions.

## frf_sweep
```@docs
frf_sweep
```

## frf_sweep Example
The following example illustrates how to use the function to obtain both an 
ascending and descending solution to Duffing's equation.
```julia
# Duffing's Equation
function duffing_eqn(du, u, p, omega, t)
    du[1] = u[2]
    du[2] = gamma * cos(omega * t) - delta * u[2] - alpha * u[1] - beta * u[1]^3
end

# Frequency Points
f1 = 0.5:0.05:2.0
f2 = 2.0:-0.05:0.5

# Perform the sweep
sweep1 = frf_sweep(
    duffing_eqn,
    [0.0, 0.0],
    f1
)
sweep2 = frf_sweep(
    duffing_eqn,
    [0.0, 0.0],
    f2
)
```
The solution returned from frf_sweep is an M-by-N matrix where M is the number of frequency points and N is the number of equations defined.  The solution is complex-valued such that each column supplies both magnitude and phase information for each equation.  For the Duffing model utilized above, the frequency response of the first equation is as follows.

![](Duffing.png?raw=true)

## Chirp Routines
The FrequencySweep.jl library also exposes routines to compute variable frequency signals often referred to as [chirp signals](https://en.wikipedia.org/wiki/Chirp).  The library exposes routines to compute the signal, its first time derivative, and its second time derivative.

```@docs
linear_chirp
exponential_chirp
hyperbolic_chirp
f
df
df2
```
## Chirp Example
The following example illustrates the use of the chirp routines for the same Duffing model used in the frf_sweep example.
```julia
# Define a linear chirp
lc = linear_chirp(
    1.0,
    0.5,
    2.0,
    500.0,
    0.0
)

# Duffing Model Parameters
# x" + delta * x' + alpha * x + beta * x^3 = gamma * cos(omega * t)
alpha = 1.0
gamma = 1.0
delta = 0.1
beta = 0.05

# Duffing's Equation
function duffing_eqn(du, u, p, t)
    du[1] = u[2]
    du[2] = gamma * f(lc, t) - delta * u[2] - alpha * u[1] - beta * u[1]^3
end

# Solve the model
prob = ODEProblem(duffing_eqn, [0.0, 0.0], [0.0, lc.time_span])
sol = solve(prob)
```
![](DuffingLinearChirp.png?raw=true)
