# The purpose of this example is to illustrate the use of a linear_chirp to 
# drive the Duffing oscillator.

using Plots
using LaTeXStrings
using DifferentialEquations
using FrequencySweep

# Define a linear chirp
lc = linear_chirp(
    1.0,
    0.5,
    2.0,
    500.0,
    0.0
)

# Duffing Model Parameters
# x" + delta * x' + alpha * x + beta * x^3 = gamma * cos(omega * t)
alpha = 1.0
gamma = 1.0
delta = 0.1
beta = 0.05

# Duffing's Equation
function duffing_eqn(du, u, p, t)
    du[1] = u[2]
    du[2] = gamma * f(lc, t) - delta * u[2] - alpha * u[1] - beta * u[1]^3
end

# Solve the model
prob = ODEProblem(duffing_eqn, [0.0, 0.0], [0.0, lc.time_span])
sol = solve(prob)

# Plot the solution
plt1 = plot(
    sol,
    vars = (0, 1),
    xlabel = L"t",
    ylabel = L"x",
    label = false
)
plt2 = plot(
    sol.t, f.(lc, sol.t),
    xlabel = L"t",
    ylabel = L"f",
    label = false
)
plt = plot(
    plt1, plt2,
    layout = (2, 1)
)
savefig(plt, "examples/plots/DuffingLinearChirp.png")