# The purpose of this example is to highlight the use of the frf_sweep
# function using Duffing's model.

using Plots
using LaTeXStrings
using DSP
using FrequencySweep

# Duffing Model Parameters
# x" + delta * x' + alpha * x + beta * x^3 = gamma * cos(omega * t)
alpha = 1.0
gamma = 1.0
delta = 0.1
beta = 0.05

# Duffing's Equation
function duffing_eqn(du, u, p, omega, t)
    du[1] = u[2]
    du[2] = gamma * cos(omega * t) - delta * u[2] - alpha * u[1] - beta * u[1]^3
end
# Analytical Solution - computes frequency values given an amplitude
function leg1(z)
    arg = 4 * gamma^2 - 3 * beta * delta^2 * z^4 + 
        (delta^2 - 4 * alpha) * delta^2 * z^2
    s = arg < 0.0 ? 
        NaN : 
        (2 * sqrt(arg) + z * (3 * beta * z^2 - 2 * delta^2 + 4 * alpha)) / (4 * z)
    return s < 0.0 ? NaN : sqrt(s)
end
function leg2(z)
    arg = 4 * gamma^2 - 3 * beta * delta^2 * z^4 + 
        (delta^2 - 4 * alpha) * delta^2 * z^2
    s = arg < 0.0 ?
        NaN :
        (-2 * sqrt(arg) + z * (3 * beta * z^2 - 2 * delta^2 + 4 * alpha)) / (4 * z)
    return s < 0.0 ? NaN : sqrt(s)
end

# Plot both legs of the actual solution
z = 0.5:0.05:10.0
magplt = plot(
    leg1.(z), z,
    label = "Analytical",
    lw = 2,
    lc = :steelblue,
    xlabel = L"\omega",
    ylabel = "X",
    legend = :topleft,
    xlims = (0.0, 2.0)
)
plot!(
    magplt,
    leg2.(z), z,
    label = false,
    lw = 2,
    lc = :steelblue
)

# Frequency Points
f1 = 0.5:0.05:2.0
f2 = 2.0:-0.05:0.5

# Perform the sweep
sweep1 = frf_sweep(
    duffing_eqn,
    [0.0, 0.0],
    f1
)
sweep2 = frf_sweep(
    duffing_eqn,
    [0.0, 0.0],
    f2
)

# Plot the results
plot!(
    magplt,
    sweep1.frequency, abs.(sweep1.response[:,1]),
    label = "Ascending",
    markershape = :circle,
    lc = :green,
    markercolor = :green
)
plot!(
    magplt,
    sweep2.frequency, abs.(sweep2.response[:,1]),
    label = "Descending",
    markershape = :xcross,
    lc = :orange,
    markercolor = :orange
)

phaseplt = plot(
    sweep1.frequency, unwrap(angle.(sweep1.response[:,1])),
    label = false,
    markershape = :circle,
    xlabel = L"\omega",
    ylabel = L"\phi",
    lc = :green,
    markercolor = :green,
    xlims = (0.0, 2.0)
)
plot!(
    sweep2.frequency, unwrap(angle.(sweep2.response[:,1])),
    label = false,
    markershape = :xcross,
    lc = :orange,
    markercolor = :orange
)

plt = plot(
    magplt, phaseplt,
    layout = (2, 1)
)
savefig(plt, "examples/plots/Duffing.png")