module FrequencySweep

export chirp,
       linear_chirp,
       exponential_chirp,
       hyperbolic_chirp,
       frf_sweep,
       f,
       df,
       df2;

    """
    A chirp is a signal whose frequency changes as a function of time.  The 
    chirp type is simply an abstract type allowing definition of various types 
    (e.g. linear, exponential, hyperbolic, etc.).
    """
    abstract type chirp end;

    Base.length(::chirp) = 1
    Base.size(::chirp) = ()
    Base.getindex(x::chirp, i) = x
    Base.iterate(x::chirp, state = 1) = state > length(x) ? nothing : (x, state = state + 1)

    """
    ## Description
    A linear chirp is a chirp signal whose frequency varies linearly with time.
    A linear chirp is given by the following function.
    \$x(t) = X \\sin \\left( \\left( \\frac{ \\left(f_{1} - f_{0} \\right) t^2}{2 t_{span}} + f_{0} t \\right) + \\phi \\right)\$
    
    ## Members
    - *amplitude:* The amplitude of the signal.
    - *start_frequency:* The initial frequency with units of rad/s, assuming time units of seconds of course.
    - *end_frequency:* The final frequency with units of rad/s, assuming time units of seconds of course.
    - *time_span:* The time duration over which the frequency will transition between the starting and ending frequencies.
    - *phase:* The phase lag of the signal, in radians.
    """
    struct linear_chirp <: chirp
        amplitude::Real
        start_frequency::Real
        end_frequency::Real
        time_span::Real
        phase::Real
    end

    """
    ## Description
    An exponential chirp is a chirp signal whose frequency varies exponentially
    with time.  An exponential chirp is given by the following function.
    
    \$x(t) = X \\sin \\left( f_{0} \\left( \\frac{k^t - 1}{\\ln (k)} \\right) + \\phi \\right)\$
    
    Where

    \$k = \\left( \\frac{f_1}{f_0} \\right)^{1/t_{span}}\$
    
    ## Members
    - *amplitude:* The amplitude of the signal.
    - *start_frequency:* The initial frequency with units of rad/s, assuming time units of seconds of course.
    - *end_frequency:* The final frequency with units of rad/s, assuming time units of seconds of course.
    - *time_span:* The time duration over which the frequency will transition between the starting and ending frequencies.
    - *phase:* The phase lag of the signal, in radians.
    """
    struct exponential_chirp <: chirp
        amplitude::Real
        start_frequency::Real
        end_frequency::Real
        time_span::Real
        phase::Real
    end

    """
    ## Description
    A hyperbolic chirp is a chirp signal whose frequency varies hyperbolically
    with time.  A hyperbolic chirp is given by the following function.

    \$x(t) = X \\sin \\left( \\alpha \\ln \\left( 1 - \\beta t \\right) + \\phi \\right)\$
    
    Where

    \$\\alpha = -\\frac{f_0 f_1 t_{span}}{f_1 - f_0}\$

    and

    \$\\beta = \\frac{f_1 - f_0}{f_1 t_{span}}\$
    
    ## Members
    - *amplitude:* The amplitude of the signal.
    - *start_frequency:* The initial frequency with units of rad/s, assuming time units of seconds of course.
    - *end_frequency:* The final frequency with units of rad/s, assuming time units of seconds of course.
    - *time_span:* The time duration over which the frequency will transition between the starting and ending frequencies.
    - *phase:* The phase lag of the signal, in radians.
    """
    struct hyperbolic_chirp <: chirp
        amplitude::Real
        start_frequency::Real
        end_frequency::Real
        time_span::Real
        phase::Real
    end

    include("FrequencySweepCore.jl")
    include("FrequencySweepChirp.jl")
end
