# FrequencySweepChirp.jl

"""
rst = f(c, t)

## Description
Evaluates a linear chirp signal at time t.

## Arguments
- *c:* The linear_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function f(c::linear_chirp, t::Real)
    p = (c.end_frequency - c.start_frequency) / (2 * c.time_span)
    c.amplitude * sin((p * t + c.start_frequency) * t + c.phase)
end

"""
rst = df(c, t)

## Description
Evaluates the first time derivative of a linear chirp signal at time t.

## Arguments
- *c:* The linear_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function df(c::linear_chirp, t::Real)
    p = (c.end_frequency - c.start_frequency) / (2 * c.time_span)
    c.amplitude * (2 * p * t + c.start_frequency) * cos((p * t + c.start_frequency) * t + c.phase)
end

"""
rst = df2(c, t)

## Description
Evaluates the second time derivative of a linear chirp signal at time t.

## Arguments
- *c:* The linear_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function df2(c::linear_chirp, t::Real)
    p = (c.end_frequency - c.start_frequency) / (2 * c.time_span)
    arg = (p * t + c.start_frequency) * t + c.phase
    c.amplitude * (
        -(2 * p * t + c.start_frequency)^2 * sin(arg) + 
        2 * p * cos(arg)
    )
end

# ------------------------------------------------------------------------------
"""
rst = f(c, t)

## Description
Evaluates an exponential chirp signal at time t.

## Arguments
- *c:* The exponential_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function f(c::exponential_chirp, t::Real)
    k = (c.end_frequency / c.start_frequency)^(1 / c.time_span)
    c.amplitude * sin(c.phase + c.start_frequency * (k^t - 1) / log(k))
end

"""
rst = df(c, t)

## Description
Evaluates the first time derivative of an exponential chirp signal at time t.

## Arguments
- *c:* The exponential_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function df(c::exponential_chirp, t::Real)
    k = (c.end_frequency / c.start_frequency)^(1 / c.time_span)
    c.amplitude * c.start_frequency * k^t * cos(c.phase + c.start_frequency * (k^t - 1) / log(k))
end

"""
rst = df2(c, t)

## Description
Evaluates the second time derivative of an exponential chirp signal at time t.

## Arguments
- *c:* The exponential_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function df2(c::exponential_chirp, t::Real)
    k = (c.end_frequency / c.start_frequency)^(1 / c.time_span)
    arg = c.phase + c.start_frequency * (k^t - 1) / log(k)
    c.amplitude * c.start_frequency * (
        k^t * log(k) * cos(arg) -
        c.start_frequency * k^(2 * t) * sin(arg)
    )
end

# ------------------------------------------------------------------------------
"""
rst = f(c, t)

## Description
Evaluates a hyperbolic chirp signal at time t.

## Arguments
- *c:* The hyperbolic_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function f(c::hyperbolic_chirp, t::Real)
    deltaF = c.end_frequency - c.start_frequency
    a = -c.start_frequency * c.end_frequency * c.time_span / deltaF
    b = deltaF / (c.end_frequency * c.time_span)
    c.amplitude * sin(a * log(1 - b * t) + c.phase)
end

"""
rst = df(c, t)

## Description
Evaluates the first time derivative of a hyperbolic chirp signal at time t.

## Arguments
- *c:* The hyperbolic_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function df(c::hyperbolic_chirp, t::Real)
    deltaF = c.end_frequency - c.start_frequency
    a = -c.start_frequency * c.end_frequency * c.time_span / deltaF
    b = deltaF / (c.end_frequency * c.time_span)
    c.amplitude * a * b * cos(a * log(1 - b * t) + c.phase) / (b * t - 1)
end

"""
rst = df2(c, t)

## Description
Evaluates the second time derivative of a hyperbolic chirp signal at time t.

## Arguments
- *c:* The hyperbolic_chirp instance.
- *t:* The time at which to evaluate the function.
"""
function df2(c::hyperbolic_chirp, t::Real)
    deltaF = c.end_frequency - c.start_frequency
    a = -c.start_frequency * c.end_frequency * c.time_span / deltaF
    b = deltaF / (c.end_frequency * c.time_span)
    arg = a * log(1 - b * t) + c.phase
    -c.amplitude * a * b^2 * (a * sin(arg) + cos(arg)) / ((b * t - 1)^2)
end


# ------------------------------------------------------------------------------
