using FrequencySweep
using Test
using DSP

@testset "FrequencySweep.jl" begin
    @testset "Spectral" begin
        # ----------------------------------------------------------------------
        # COMPUTE_XFRM_LENGTH
        n1 = FrequencySweep.compute_xfrm_length(500)
        @test isequal(n1, 251)

        n2 = FrequencySweep.compute_xfrm_length(501)
        @test isequal(n2, 251)

        # ----------------------------------------------------------------------
        # ADD_SEGMENT! - FIRST TIME TEST
        t = 0:1e-3:1.0
        X = 2.5
        freq = 50.0
        x = X .* sin.((2 * pi * freq) .* t)

        xfrm = zeros(
            ComplexF64,
            floor(Int64, FrequencySweep.compute_xfrm_length(length(t)))
        )
        FrequencySweep.add_segment!(
            hanning,
            length(xfrm),
            x,
            xfrm,
            true
        )

        tol = 0.02
        maxX = maximum(abs.(xfrm))
        @test isless(abs(X - maxX), tol)

        # ----------------------------------------------------------------------
        # ADD_SEGMENT! - SECOND TIME TEST
        FrequencySweep.add_segment!(
            hanning,
            length(xfrm),
            x,
            xfrm,
            false
        )

        maxX1a = maximum(abs.(xfrm))
        @test isless(abs(2 * X - maxX1a), tol)

        # ----------------------------------------------------------------------
        # OVERLAP_SEGMENT - OVERLAP LENGTH TEST
        xfrm2 = zeros(
            ComplexF64,
            500
        )
        count = FrequencySweep.overlap_segment(
            hanning,
            length(xfrm2),
            x,
            xfrm2,
            true,
            0
        )

        maxX2 = maximum(abs.(xfrm2))
        @test isless(abs(X - maxX2 / count), tol)

        # ----------------------------------------------------------------------
        # OVERLAP_SEGMENT - PAD WITH ZEROS TEST
        xfrm3 = zeros(
            ComplexF64,
            length(t)
        )
        count = FrequencySweep.overlap_segment(
            hanning,
            length(xfrm3),
            x,
            xfrm3,
            true,
            0
        )
        xfrm3 = xfrm3 ./ count  # need to average

        @test isequal(count, 1) # only 1 time through for a pad-with-zeros

        maxX3 = maximum(abs.(xfrm3))
        @test isless(abs(X - maxX3), tol)

        # ----------------------------------------------------------------------
        # SPECTRUM - OVERLAP TEST
        X1 = FrequencySweep.spectrum(
            hanning,
            500,
            x
        )
        check1 = maximum(abs.(X1))
        @test isless(abs(check1 - X), tol)
    end

    @testset "TimeVectorTests" begin
        # Define vector parameters
        pointsPerCycle = 100
        omega = 50.0
        nTransientCycles = 10
        nSteadyCycles = 10
        nCycles = nTransientCycles + nSteadyCycles
        npts = pointsPerCycle * nSteadyCycles
        period = 2 * pi / omega
        dt = period / (pointsPerCycle - 1)

        # Generate a reference time vector
        tReference = zeros(Float64, npts)
        tReference[1] = nTransientCycles * period + dt
        for i in 2:npts
            tReference[i] = tReference[i-1] + dt
        end

        # Allocate a time array for testing
        t = zeros(Float64, npts)

        # Run the function to test
        FrequencySweep.fill_time_vector!(
            nTransientCycles,
            pointsPerCycle,
            omega,
            t
        )

        # Compare the vectors element by element
        @test isequal(t, tReference)
    end

    @testset "LinearChirp" begin
        # Define a linear chirp
        lc = linear_chirp(
            0.5,
            1.0,
            100.0,
            10.0,
            0.0
        )

        # Define the linear chirp function so we can compare
        function fun(t)
            c = (lc.end_frequency - lc.start_frequency) / (2 * lc.time_span)
            lc.amplitude * sin((c * t + lc.start_frequency) * t + lc.phase)
        end
        function diff_fun(t)
            c = (lc.end_frequency - lc.start_frequency) / (2 * lc.time_span)
            lc.amplitude * (2 * c * t + lc.start_frequency) * cos((c * t + lc.start_frequency) * t + lc.phase)
        end
        function diff2_fun(t)
            c = (lc.end_frequency - lc.start_frequency) / (2 * lc.time_span)
            arg = (c * t + lc.start_frequency) * t + lc.phase
            lc.amplitude * (-(2 * c * t + lc.start_frequency)^2 * sin(arg) + 2 * c * cos(arg))
        end

        # Define a series of time points
        t = 0.0:0.005:lc.time_span

        # Compare results
        xAns = fun.(t)
        x = f.(lc, t)
        @test isequal(xAns, x)

        dxAns = diff_fun.(t)
        dx = df.(lc, t)
        @test isequal(dxAns, dx)

        dx2Ans = diff2_fun.(t)
        dx2 = df2.(lc, t)
        @test isequal(dx2Ans, dx2)
    end

    @testset "ExponentialChirp" begin
        # Define an exponential chirp
        ec = exponential_chirp(
            0.5,
            1.0,
            100.0,
            10.0,
            0.0
        )

        # Define the chirp function so we can compare
        function fun(t)
            k = (ec.end_frequency / ec.start_frequency)^(1 / ec.time_span)
            ec.amplitude * sin(ec.phase + ec.start_frequency * (k^t - 1) / log(k))
        end
        function diff_fun(t)
            k = (ec.end_frequency / ec.start_frequency)^(1 / ec.time_span)
            ec.amplitude * ec.start_frequency * k^t * cos(ec.phase + ec.start_frequency * (k^t - 1) / log(k))
        end
        function diff2_fun(t)
            k = (ec.end_frequency / ec.start_frequency)^(1 / ec.time_span)
            arg = ec.phase + ec.start_frequency * (k^t - 1) / log(k)
            ec.amplitude * ec.start_frequency * (
                k^t * log(k) * cos(arg) -
                ec.start_frequency * k^(2 * t) * sin(arg)
            )
        end

        # Define a series of time points
        t = 0.0:0.005:ec.time_span

        # Compare results
        xAns = fun.(t)
        x = f.(ec, t)
        @test isequal(xAns, x)

        dxAns = diff_fun.(t)
        dx = df.(ec, t)
        @test isequal(dxAns, dx)

        dx2Ans = diff2_fun.(t)
        dx2 = df2.(ec, t)
        @test isequal(dx2Ans, dx2)
    end

    @testset "HyperbolicChirp" begin
        # Define a hyperbolic chirp
        hc = hyperbolic_chirp(
            0.5,
            1.0,
            100.0,
            10.0,
            0.0
        )

        # Define the chirp function so we can compare
        function fun(t)
            deltaF = hc.end_frequency - hc.start_frequency
            a = -hc.start_frequency * hc.end_frequency * hc.time_span / deltaF
            b = deltaF / (hc.end_frequency * hc.time_span)
            arg = a * log(1 - b * t) + hc.phase
            hc.amplitude * sin(arg)
        end
        function diff_fun(t)
            deltaF = hc.end_frequency - hc.start_frequency
            a = -hc.start_frequency * hc.end_frequency * hc.time_span / deltaF
            b = deltaF / (hc.end_frequency * hc.time_span)
            arg = a * log(1 - b * t) + hc.phase
            hc.amplitude * a * b * cos(arg) / (b * t - 1)
        end
        function diff2_fun(t)
            deltaF = hc.end_frequency - hc.start_frequency
            a = -hc.start_frequency * hc.end_frequency * hc.time_span / deltaF
            b = deltaF / (hc.end_frequency * hc.time_span)
            arg = a * log(1 - b * t) + hc.phase
            -hc.amplitude * a * b^2 * (a * sin(arg) + cos(arg)) / ((b * t - 1)^2)
        end

        # Define a series of time points
        t = 0.0:0.005:hc.time_span

        # Compare results
        xAns = fun.(t)
        x = f.(hc, t)
        @test isequal(xAns, x)

        dxAns = diff_fun.(t)
        dx = df.(hc, t)
        @test isequal(dxAns, dx)

        dx2Ans = diff2_fun.(t)
        dx2 = df2.(hc, t)
        @test isequal(dx2Ans, dx2)
    end
end
